#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <algorithm>
#include <vector>
#include <sstream>
#include <stdexcept>
#include <pthread.h>
#include <signal.h>
#include <errlog.h>

#include "dome.h"
#include "defnames.h"

pthread_mutex_t Dome::Lock::mtx = PTHREAD_MUTEX_INITIALIZER;
Dome_CMD Dome::cmd = { std::string(program_cfg), false, false, false };
Dome* Dome::self_ptr = 0;
const int Dome::initLifeSec = 32;
const int Dome::parkLifeSec = 32;
pthread_t Dome::thread_id;

Dome::Dome(): Domecfg( cmd.cfgfile ), deviceDome( Domecfg ),
    closeDome( Domecfg, deviceDome ), openDome( Domecfg, deviceDome ), moveDome( Domecfg, deviceDome ) {
    tableMode["dome"]   = &openDome;
    tableMode["sideA"]  = &openDome;
    tableMode["sideB"]  = &openDome;
    tableDome["open"]   = &openDome;
    tableDome["close"]  = &closeDome;
//    std::cerr << "Constructor Dome\n";
}

void Dome::create() {
    Lock lock;
    ErrlogSingl::instance().reopen();
    if(self_ptr == 0) self_ptr = new Dome();
}

void Dome::destroy() {
    Lock lock;       // To prevent commands execution
    if(self_ptr == 0) return;
    delete self_ptr; // during Dome destruction
    self_ptr = 0;
}

Dome* Dome::device_ptr() {
    Lock lock;
    if( self_ptr == 0 ) throw ErrPark(ERR_INI_OBJECT, "Dome object");
    return self_ptr;
}

int Dome::specify( const std::vector<std::string>& opts ) {
    for( unsigned int j = 0; j < opts.size(); j++ ) {
        if(tableMode.find(opts.at(j)) == tableMode.end()) throw ErrSyn( ERR_PROT_WRKWD, opts.at(j) );
        the = tableMode[opts.at(j)];
    }
    return the->life();
}

int  Dome::stop( const std::vector<std::string>& opts ) {
    if( !opts.empty() && opts.at(0) != "now" ) throw ErrSyn( ERR_PROT_WRKWD, opts.at(0) );
    pthread_kill( thread_id, SIGUSR1 );       //
//    if( !opts.empty() )
    {                                         // stop == stop now
        the->stop_request();
        return 1;
    }
    return the->life();
}

typedef void(Dome::*SETF_T)(const std::string&);
typedef std::pair<const std::string, SETF_T> SET_PAIR;

void Dome::set(const std::string& key, const std::string& val) {
    static SET_PAIR set_kwd[] =  {
        SET_PAIR ("sideA",  &Dome::setSideAaction),
        SET_PAIR ("sideB",  &Dome::setSideBaction),
        SET_PAIR ("dome",   &Dome::setDomeAction)
    };
    static std::map<const std::string, SETF_T, Cmpi> setf(set_kwd, set_kwd + sizeof(set_kwd)/sizeof(set_kwd[0]));
    if( setf.find(key) == setf.end()) throw ErrSyn( ERR_PROT_WRKWD, val);
    std::string s(val);
    std::transform( s.begin(), s.end(), s.begin(), std::ptr_fun(::tolower) );
    (this->*setf[key])(s);
}

double Dome::defineAction( const std::string &s ) {
    double target;
    std::istringstream str(s);
    str >> target;
    if( !str.fail() && target > 0.0 && target < 1.0 ) return target;
    return -1;
}

void Dome::setDomeAction( const std::string &s ) {
    Mode::both();
    if( tableDome.find( s ) == tableDome.end() ) throw ErrSyn( ERR_PROT_WRKWD, s );
    tableMode["dome"] = tableDome[s];
}

void Dome::setSideAaction( const std::string &s ) {
    Mode::segmA();
    if( defineAction(s) > 0.0 ) {
        moveDome.set_target( defineAction(s) );                   // Set target here
        tableMode["sideA"] =  &moveDome;
        return;
    }
    if( tableDome.find( s ) == tableDome.end() ) throw ErrSyn( ERR_PROT_WRKWD, s );
    tableMode["sideA"] = tableDome[s];
}

void Dome::setSideBaction( const std::string &s ) {
    Mode::segmB();
    if( defineAction(s) > 0.0 ) {
        moveDome.set_target( defineAction(s) );                   // Set target here
        tableMode["sideB"] =  &moveDome;
        return;
    }
    if( tableDome.find( s ) == tableDome.end() ) throw ErrSyn( ERR_PROT_WRKWD, s );
    tableMode["sideB"] = tableDome[s];
}

typedef std::string(Dome::*GETF_T)();
typedef std::pair<const std::string, GETF_T> GET_PAIR;

std::string Dome::get(const std::string& key) {
    static GET_PAIR get_kwd[] =  {
        GET_PAIR ("dome",   &Dome::dome_status ),
        GET_PAIR ("position",   &Dome::side_position )
    };
    static std::map<const std::string, GETF_T, Cmpi> getf(get_kwd, get_kwd + sizeof(get_kwd)/sizeof(get_kwd[0]));
    if( getf.find(key) == getf.end()) throw ErrSyn( ERR_PROT_WRKWD, key);
    return (this->*getf[key])();
}

std::string Dome::dome_status() {
    return "DOME=" + deviceDome.domestatus();
}

std::string Dome::side_position() {
    std::ostringstream str;
    str << std::fixed << std::setprecision(2) << "SIDE_A=" << deviceDome.ahaven_sideA_position() << " SIDE_B=" << deviceDome.ahaven_sideB_position();
    return str.str();
}
