#define __USE_GNU
#include <stdio.h>
#include <stdlib.h>
#include <cmath>
#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>

#include "astrotime.h"

AstroDateTime::AstroDateTime(){
    gettimeofday( &hightime, NULL );
    gmtime_r( &hightime.tv_sec, &UTstart );
    UTstart.tm_hour = UTstart.tm_min = UTstart.tm_sec = 0;
    start_midnight = timegm( &UTstart );
    UTstart.tm_year += 1900;
    UTstart.tm_mon += 1;
    at_julian = julian( UTstart.tm_year, UTstart.tm_mon, UTstart.tm_mday );
    at_s0 = s0();
    realt = true;
};

AstroDateTime::AstroDateTime( int y, int m, int d ) {
    hightime.tv_sec = hightime.tv_usec = 0;
    UTstart.tm_year = y - 1900;
    UTstart.tm_mon = m - 1;
    UTstart.tm_mday = d;
    UTstart.tm_hour = UTstart.tm_min = UTstart.tm_sec = 0;
    start_midnight = timegm( &UTstart );
    UTstart.tm_year += 1900;
    UTstart.tm_mon += 1;
    at_julian = julian( UTstart.tm_year, UTstart.tm_mon, UTstart.tm_mday );
    at_s0 = s0();
    realt = false;
//    std::cerr << "Nes astro constructor: " << at_julian << " " << at_s0 << std::endl;
};

void AstroDateTime::current( int h, int m, double s ) {
    realt = false;
    hightime.tv_sec = start_midnight + h*3600 + m*60 + int(s);
    hightime.tv_usec = int((s-floor(s))*1e6);
    current( 0 );
}

void AstroDateTime::current( int mode ) {
    if (realt) gettimeofday( &hightime, NULL );
/*    if((long)hightime.tv_usec >= 999500L ) {
        hightime.tv_sec++;
        hightime.tv_usec = 0;
    }*/
    gmtime_r( &hightime.tv_sec, &UTC );
    int lday  = mode ? 0 : (hightime.tv_sec - start_midnight)/(24*60*60);
    UTC.tm_year = mode ? UTC.tm_year + 1900 : UTstart.tm_year;
    UTC.tm_mon = mode ? UTC.tm_mon + 1 : UTstart.tm_mon;
    UTC.tm_mday = mode ? UTC.tm_mday : UTstart.tm_mday;
    UTC.tm_hour = UTC.tm_hour+24*lday;
//    UTC_msec  = (int)(hightime.tv_usec+500)/1000 ;
    UTC_msec  = (int)hightime.tv_usec/1000 ;
}

void AstroDateTime::local(){
    if (realt) gettimeofday( &hightime, NULL );
/*    if((long)hightime.tv_usec >= 999500L ) {
        hightime.tv_sec++;
        hightime.tv_usec = 0;
    }*/
    localtime_r( &hightime.tv_sec, &UTC );
    UTC.tm_year += 1900;
    UTC.tm_mon += 1;
}

int AstroDateTime::julian( int year, int month, int day ) {
    return ::julian( year, month, day );
}

std::string AstroDateTime::strTime( int pr ) {
    std::ostringstream timestr;
    timestr << std::setfill('0') << std::setw(2) << UTC.tm_hour << ':'
            << std::setw(2) << UTC.tm_min  << ':' << std::setw(2) << UTC.tm_sec;
    if( pr > 0 ) {
        int fracs = (pr==1)? 100 : ( (pr==2)? 10 : 1  );
        timestr << '.' << std::setw( pr )  << UTC_msec/fracs;
    }
    return timestr.str();
}

std::string AstroDateTime::strDate( void ) {
    std::ostringstream datestr;
    datestr << std::setfill('0') << std::setw(4) << UTC.tm_year  << '-'
            << std::setw(2) << UTC.tm_mon << '-'
            << std::setw(2) << UTC.tm_mday;
    return datestr.str();
}

std::string AstroDateTime::strFileName() {
    std::ostringstream datestr;
    if (realt) gettimeofday( &hightime, NULL );
    hightime.tv_sec -= 12*60*60;    //  
    localtime_r( &hightime.tv_sec, &LT );
    datestr << std::setfill('0') << std::setw(2) << LT.tm_year - 100
            << std::setw(2) << LT.tm_mon + 1
            << std::setw(2) << LT.tm_mday;
    return datestr.str();
}

/* Calculation of sidereal time in the Greewhich midnigth (in days) */
double AstroDateTime::s0( int jd ) {
    return ::s0( jd );
}

double AstroDateTime::currentSec() {
    if (realt) gettimeofday( &hightime, NULL );
    return (double)(hightime.tv_sec - start_midnight) + hightime.tv_usec/1000000.0;
}

/* Calculation of the local sidereal time  (in days) */
double AstroDateTime::sidTime( double longitude ) {
    double  stg = at_s0 + 1.0027379093*currentSec()/86400.0 + longitude;
    return stg - floor(stg);
}

double AstroDateTime::rsidTime( double longitude ) {
    return 2*M_PI*sidTime( longitude/(2*M_PI) );
}
