#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <valarray>
#include <iostream>
#include <iomanip>
#include <vector>
#include <numeric>
#include <signal.h>
#include <unistd.h>
#include "modes.h"

const double spar_time=1.0;

const std::string   sect_operation("Operations");
const std::string   subs_close("Close");
const std::string   subs_open("Open");
bool  Mode::sideA = true;
bool  Mode::sideB = true;

Mode::Mode( Config &cf, Device &cm ) {
    cfg = &cf;
    ddv = &cm;
}

Close::Close( Config &cf, Device &cm ) : Mode( cf, cm ) {
    up_time = ddv->ahaven_time();
}

void  Close::action() {
    break_mode = false;
    ErrlogSingl::instance().write( "Start to be dome close" );
    std::cerr << "Close : " << sideA << "  " << sideB << std::endl;
    if( sideA && sideB ) {
        ddv->ahaven_close_b();                  //
        usleep( 1000000 );
        ddv->ahaven_close_a();                  // If close_b non-stopped
    }
    if( sideA && !sideB ) {
        ddv->ahaven_close_a();                  // If close_b non-stopped
    }
    if( !sideA && sideB ) {
        ddv->ahaven_close_b();                  //
    }
    ddv->ahaven_is_closed( sideA, sideB );
}

int  Close::life() {
    return (int)up_time;
}

Open::Open( Config &cf, Device &cm ) : Mode( cf, cm ) {
    up_time = ddv->ahaven_time();
}

void Open::action() {
    break_mode = false;
    ErrlogSingl::instance().write( "Start to be dome open" );
    std::cerr << "Open : " << sideA << "  " << sideB << std::endl;
    if( sideA && sideB ) {
        ddv->ahaven_open_a();
        usleep( 1000000 );
        ddv->ahaven_open_b();
    }
    if( sideA && !sideB ) {
        ddv->ahaven_open_a();
    }
    if( !sideA && sideB ) {
        ddv->ahaven_open_b();
    }
    ddv->ahaven_is_opened( sideA, sideB );
}

int Open::life() {
    return (int)up_time;
}


Move::Move( Config &cf, Device &cm ) : Mode( cf, cm ) {
    up_time = ddv->ahaven_time();
}

void Move::action() {
    break_mode = false;
    ErrlogSingl::instance().write( "Start to be dome move" );
    std::cerr << "Move  : " << sideA << "  " << sideB << std::endl;
    if( sideA && !sideB ) {
        ddv->ahaven_move_a( target );
    }
    if( !sideA && sideB ) {
        ddv->ahaven_move_b( target );
    }
}

int Move::life() {
    return (int)up_time;
}


