#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <valarray>
#include <iostream>
#include <iomanip>
#include <vector>
#include <numeric>
#include <errlog.h>
#include <client_handler.h>

const int spar_time = 5;
int ClientHandler::start_id = 1000;
                        //      
ClientHandler::ClientHandler( ClientSocket::AddrType ip ) : id_numb( start_id ) {
    ips = ip;
    client = std::auto_ptr<Client>( new Client( ip,  this ) );
    if( !client->isConnected() ) {
        std::ostringstream ws;
        ws << "Connection to " << ip << " refused";
        throw  std::runtime_error( ws.str() );
    }
    start_id += 1000;
}

void ClientHandler::reconnect() {   //    
        client.reset( new Client( ips,  this ) );
        sleep(5);
        if( client->isConnected() ) return;
        std::ostringstream ws;
        ws << ips;
        if( reconn_count++ > max_reconn ) throw ErrFat( ERR_SOCK_BAD, "Server " + ws.str() + " doesn't work" );
        else throw std::runtime_error( "Reconnect to " + ws.str() + " refused" );
}

int ClientHandler::parse_structure( std::string& buf ) {
    ErrlogSingl::instance().write_if_test( buf );
    std::vector<std::string> answer_items;
    Tool::parse( buf.c_str(), answer_items );
    if( answer_items.at(0) != id() ) {
        ErrlogSingl::instance().write( "Protocol error: identification not match" );
        return -1;
    }
    success = ( answer_items.at(1) == "OK" ) ? true : false;
    answer_vals.clear();
    std::string::size_type i;
    for( unsigned int j = 2; j < answer_items.size(); j++ ) {
        if( (i =  answer_items.at(j).find("=",0)) != std::string::npos ) {
            answer_vals[answer_items.at(j).substr( 0, i )] = answer_items.at(j).substr( i+1 );
        } else {
            answer_vals[answer_items.at(j)] = "";
        }
    }
    if( status( "ERSYN" ) ) throw ErrFat( ERR_ALG, "Server error: " + answer_vals["status"] );
    return 0;
}

typedef void(ClientHandler::*SELF_T)( std::string );
typedef std::pair<std::string, SELF_T> PT_PAIR;

bool ClientHandler::out( const std::string& commstr ) {
    try {
        if( !client->isConnected() ) {
            ErrlogSingl::instance().write( "Try reconnect" );
            reconnect();
        }
        client->sendRequest( commstr );
        return true;
    }
    catch( const std::exception& e ) {
        return false;
    }
}

bool ClientHandler::command( const std::string& commstr ) {
    nonblocked( commstr );
    return wait_command();
}

bool ClientHandler::wait_nonbusy( const std::string& commstr ) {
    int counter = 16;
    command( commstr );
    while( status( "BUSY" ) && counter-- > 0 ) {
        usleep( 1000000 );
        command( commstr );
    }
    return status( "READY" )||status( "PARKED" );
}

void ClientHandler::nonblocked( const std::string& commstr ) {
    static PT_PAIR m[] =  {
        PT_PAIR( "init", &ClientHandler::rfunc ),
        PT_PAIR( "park", &ClientHandler::rfunc ),
        PT_PAIR( "quit", &ClientHandler::rfunc ),
        PT_PAIR( "run",  &ClientHandler::rfunc ),
        PT_PAIR( "stop", &ClientHandler::stopf ),
        PT_PAIR( "get",  &ClientHandler::gfunc ),
        PT_PAIR( "set",  &ClientHandler::sfunc )
    };
    static std::map<std::string, SELF_T, Cmpi> handf( m, m + sizeof(m)/sizeof(m[0]));
    std::string comm;
    std::istringstream ids( commstr );
    ids >> comm;
    if( handf.find( comm ) == handf.end() ) throw ErrFat( ERR_ALG, "Wrong command" );
    id_numb++;
    std::string oustr = id() + " " + commstr;
    ErrlogSingl::instance().write_if_test( oustr );
    if( !client->isConnected() ) {
        ErrlogSingl::instance().write( "Connection losses, try reconnect" );
        reconnect();
    }
    client->sendRequest( oustr, handf[comm] );
}

bool ClientHandler::wait_command() {
    while( answer_vals.find( "wait" ) != answer_vals.end() ) {
        client->waitReply( get_value<int>("wait")+spar_time );
    }
    return success;
}

void ClientHandler::rfunc( std::string buf = std::string()) {
    if( parse_structure( buf ) != 0 ) return;
}

void ClientHandler::stopf( std::string buf = std::string()) {
    if( parse_structure( buf ) != 0 ) return; //  ID
    id_numb--;
}

void ClientHandler::gfunc( std::string buf = std::string()) {
    if( parse_structure( buf ) != 0 ) return;
}

void ClientHandler::sfunc( std::string buf = std::string()) {
    if( parse_structure( buf ) != 0 ) return;
}

