/*
*   socket.cpp is a part of programm carbon
*   $Id$
*   Copyright (C) 2005-2006  Matwey V. Kornilov
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <sstream>

#include "socket.h"

IP4Addr::IP4Addr(){
}

IP4Addr::IP4Addr( const std::string& name, unsigned short port ) : m_name( name ), m_port( port ){
}

IP4Addr::~IP4Addr(){
}

const std::string& IP4Addr::getName() const {
    return m_name;
}

unsigned short IP4Addr::getPort() const {
    return m_port;
}

std::ostream& std::operator<<( std::ostream& stm, const IP4Addr& a ){
    return stm << a.getName() << ":" << a.getPort();
}

std::istream& std::operator>>( std::istream& stm, IP4Addr& a ){
    std::string addr;
    stm >> addr;
    std::string::size_type pos = 0;
    if( ( pos = addr.find(":") ) == std::string::npos ){
        a = IP4Addr( addr, 0 );
    } else {
        std::istringstream iss( addr.substr( pos+1, addr.size() ) );
        unsigned short port;
        iss >> port;
        a = IP4Addr( addr.substr(0,pos), port );
    }
    return stm;
}

template <class SocketTraits> BasicSocketBase<SocketTraits>::BasicSocketBase(): m_state(Idle){
}

template <class SocketTraits> BasicSocketBase<SocketTraits>::~BasicSocketBase() {
}

template <class SocketTraits> void BasicSocketBase<SocketTraits>::setState(SocketState state){
    if( state != m_state ) {
        onStateChanged(state);
        m_state=state;
    }
}

template <class SocketTraits> typename BasicSocketBase<SocketTraits>::SocketState BasicSocketBase<SocketTraits>::getState() const{
    return m_state;
}

template <class SocketTraits> const typename BasicSocketBase<SocketTraits>::AddrType BasicSocketBase<SocketTraits>::getPeerName() const{
    return m_pImpl->getPeerName();
}

template <class SocketTraits> const typename BasicSocketBase<SocketTraits>::AddrType BasicSocketBase<SocketTraits>::getSockName() const{
    return m_pImpl->getSockName();
}

template <class SocketTraits> BasicClientSocket<SocketTraits>::BasicClientSocket(){
    BasicSocketBase<SocketTraits>::m_pImpl=new SocketTraits();
    SocketWatcher::instance().addSocket(this);
}

template <class SocketTraits> BasicClientSocket<SocketTraits>::BasicClientSocket( ServerSocket* serv ){
    BasicSocketBase<SocketTraits>::m_pImpl=serv->popAcceptedTraits();
    SocketWatcher::instance().addSocket(this);
    this->setState(SocketBase::Connected);
}

template <class SocketTraits> BasicClientSocket<SocketTraits>::~BasicClientSocket(){
    SocketWatcher::instance().removeSocket(this);
}

template <class SocketTraits> int BasicClientSocket<SocketTraits>::hasData() const{
    return m_buf.size();
}

template <class SocketTraits> void BasicClientSocket<SocketTraits>::getData( std::string& dest, std::string::size_type beg, std::string::size_type end ){
    if(beg==0 && end==0) end=buffer().size();
    dest = m_buf.substr(beg,end);
    m_buf.erase(beg,end);
}

template <class SocketTraits> void BasicClientSocket<SocketTraits>::connect( const typename ClientSocket::AddrType& addr ){
    BasicSocketBase<SocketTraits>::m_pImpl->connect( addr );
    this->setState( SocketBase::Connecting );
}

template <class SocketTraits> void BasicClientSocket<SocketTraits>::close(){
    BasicSocketBase<SocketTraits>::m_pImpl->close();
    this->setState( SocketBase::Idle );
}

template <class SocketTraits> void BasicClientSocket<SocketTraits>::read(){
    m_buf+=BasicSocketBase<SocketTraits>::m_pImpl->read();
}

template <class SocketTraits> BasicServerSocket<SocketTraits>::BasicServerSocket(){
    BasicSocketBase<SocketTraits>::m_pImpl=new SocketTraits();
    SocketWatcher::instance().addSocket( this );
}

template <class SocketTraits> BasicServerSocket<SocketTraits>::~BasicServerSocket() {
    SocketWatcher::instance().removeSocket( this );
}

template <class SocketTraits> void BasicServerSocket<SocketTraits>::accept(){
    SocketTraits* nsock = 0;
    nsock = BasicSocketBase<SocketTraits>::m_pImpl->accept();
    if( nsock ) m_accepted.push( nsock );
}

template <class SocketTraits> int BasicServerSocket<SocketTraits>::hasAccepted() const{
    return m_accepted.size();
}

template <class SocketTraits> void BasicServerSocket<SocketTraits>::listen( const typename ServerSocket::AddrType& addr, int backlog ){
    BasicSocketBase<SocketTraits>::m_pImpl->bind( addr );
    BasicSocketBase<SocketTraits>::m_pImpl->listen( backlog );
    this->setState( SocketBase::Listening );
    accept();
}

template <class SocketTraits> void BasicServerSocket<SocketTraits>::close(){
    BasicSocketBase<SocketTraits>::m_pImpl->close();
    this->setState( SocketBase::Idle );
}

template <class SocketTraits> SocketTraits* BasicServerSocket<SocketTraits>::popAcceptedTraits(){
    SocketTraits* ret = 0;
    ret=m_accepted.top();
    m_accepted.pop();
    return ret;
}

template <class SocketTraits> BasicSocketError<SocketTraits>::BasicSocketError( const std::string& what,Error error ): std::runtime_error( what ), m_error( error ) {
}

template <class SocketTraits> typename BasicSocketError<SocketTraits>::Error BasicSocketError<SocketTraits>::code() const{
    return m_error;
}

template <class SocketTraits> void BasicSocketBase<SocketTraits>::onStateChanged( const typename SocketBase::SocketState s ){
}

template <class SocketTraits> void BasicClientSocket<SocketTraits>::onDataRecieved(){
    read();
}

template <class SocketTraits> void BasicClientSocket<SocketTraits>::onError( const SocketError& e ){
   close();
}

template <class SocketTraits> void BasicServerSocket<SocketTraits>::onAccepted(){
    accept();
}

template <class SocketTraits> void BasicServerSocket<SocketTraits>::onError( const SocketError& e ){
   close();
}

#include "linux.cpp"
#include "callback.cpp"

template class BasicSocketBase< DefinedSocketTraits >;
template class BasicClientSocket< DefinedSocketTraits >;
template class BasicServerSocket< DefinedSocketTraits >;
template class BasicSocketWatcher< DefinedSocketTraits >;
template class BasicSocketError< DefinedSocketTraits >;
